@@ -70,8 +70,8 @@ mpfr_ai1 (mpfr_ptr y, mpfr_srcptr x, mpfr_rnd_t rnd)
 
   /* Logging */
   MPFR_LOG_FUNC (
-    ("x[%Pu]=%.*Rg rnd=%d", mpfr_get_prec (x), mpfr_log_prec, x, rnd),
-    ("y[%Pu]=%.*Rg", mpfr_get_prec (y), mpfr_log_prec, y) );
+    ("x[%Pd]=%.*Rg rnd=%d", mpfr_get_prec (x), mpfr_log_prec, x, rnd),
+    ("y[%Pd]=%.*Rg", mpfr_get_prec (y), mpfr_log_prec, y) );
 
   /* Save current exponents range */
   MPFR_SAVE_EXPO_MARK (expo);
@@ -198,7 +198,7 @@ mpfr_ai1 (mpfr_ptr y, mpfr_srcptr x, mpfr_rnd_t rnd)
   /* ZIV loop */
   for (;;)
     {
-      MPFR_LOG_MSG (("Working precision: %Pu\n", wprec));
+      MPFR_LOG_MSG (("Working precision: %Pd\n", wprec));
       mpfr_set_prec (ti, wprec);
       mpfr_set_prec (tip1, wprec);
       mpfr_set_prec (x3, wprec);
@@ -257,8 +257,8 @@ mpfr_ai1 (mpfr_ptr y, mpfr_srcptr x, mpfr_rnd_t rnd)
 
       /* err is the number of bits lost due to the evaluation error */
       /* wprec-(prec+1): number of bits lost due to the approximation error */
-      MPFR_LOG_MSG (("Roundoff error: %Pu\n", err));
-      MPFR_LOG_MSG (("Approxim error: %Pu\n", wprec-prec-1));
+      MPFR_LOG_MSG (("Roundoff error: %Pd\n", err));
+      MPFR_LOG_MSG (("Approxim error: %Pd\n", wprec-prec-1));
 
       if (wprec < err + 1)
         correct_bits = 0;
@@ -342,8 +342,8 @@ mpfr_ai2 (mpfr_ptr y, mpfr_srcptr x, mpfr_rnd_t rnd)
 
   /* Logging */
   MPFR_LOG_FUNC (
-    ("x[%Pu]=%.*Rg rnd=%d", mpfr_get_prec (x),  mpfr_log_prec, x, rnd),
-    ("y[%Pu]=%.*Rg", mpfr_get_prec (y), mpfr_log_prec, y));
+    ("x[%Pd]=%.*Rg rnd=%d", mpfr_get_prec (x),  mpfr_log_prec, x, rnd),
+    ("y[%Pd]=%.*Rg", mpfr_get_prec (y), mpfr_log_prec, y));
 
   /* Save current exponents range */
   MPFR_SAVE_EXPO_MARK (expo);
@@ -431,7 +431,7 @@ mpfr_ai2 (mpfr_ptr y, mpfr_srcptr x, mpfr_rnd_t rnd)
   /* ZIV loop */
   for (;;)
     {
-      MPFR_LOG_MSG (("working precision: %Pu\n", wprec));
+      MPFR_LOG_MSG (("working precision: %Pd\n", wprec));
 
       for (j=0; j<=L; j++)
         mpfr_set_prec (z[j], wprec);
@@ -548,8 +548,8 @@ mpfr_ai2 (mpfr_ptr y, mpfr_srcptr x, mpfr_rnd_t rnd)
 
       /* err is the number of bits lost due to the evaluation error */
       /* wprec-(prec+1): number of bits lost due to the approximation error */
-      MPFR_LOG_MSG (("Roundoff error: %Pu\n", err));
-      MPFR_LOG_MSG (("Approxim error: %Pu\n", wprec - prec - 1));
+      MPFR_LOG_MSG (("Roundoff error: %Pd\n", err));
+      MPFR_LOG_MSG (("Approxim error: %Pd\n", wprec - prec - 1));
 
       if (wprec < err+1)
         correctBits = 0;
