@@ -1,6 +1,6 @@
 /* Test file for mpfr_get_str.
 
-Copyright 1999, 2001-2016 Free Software Foundation, Inc.
+Copyright 1999, 2001-2023 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -17,11 +17,9 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdlib.h>
-
 #include "mpfr-test.h"
 
 static void
@@ -78,9 +76,9 @@ check_small (void)
 
   mpfr_set_prec (x, 64);
   mpfr_set_si (x, -1, MPFR_RNDN);
-  mpfr_div_2exp (x, x, 63, MPFR_RNDN); /* x = -2^(-63) */
+  mpfr_div_2ui (x, x, 63, MPFR_RNDN); /* x = -2^(-63) */
   mpfr_add_ui (x, x, 1, MPFR_RNDN); /* x = 1 - 2^(-63) */
-  mpfr_mul_2exp (x, x, 32, MPFR_RNDN); /* x = 2^32 - 2^(-31) */
+  mpfr_mul_2ui (x, x, 32, MPFR_RNDN); /* x = 2^32 - 2^(-31) */
   s = mpfr_get_str (NULL, &e, 3, 21, x, MPFR_RNDU);
   if (strcmp (s, "102002022201221111211") || (e != 21))
     {
@@ -993,7 +991,7 @@ check_large (void)
 
   mpfr_init2 (x, 3322);
   mpfr_set_str (x, xm, 10, MPFR_RNDN);
-  mpfr_div_2exp (x, x, 4343, MPFR_RNDN);
+  mpfr_div_2ui (x, x, 4343, MPFR_RNDN);
   s = mpfr_get_str (NULL, &e, 10, 1000, x, MPFR_RNDN);
   if (s[999] != '1') /* s must be 5.04383...689071e-309 */
     {
@@ -1003,7 +1001,7 @@ check_large (void)
     }
   mpfr_free_str (s);
 
-  mpfr_mul_2exp (x, x, 4343, MPFR_RNDN);
+  mpfr_mul_2ui (x, x, 4343, MPFR_RNDN);
   s = mpfr_get_str (NULL, &e, 10, 2, x, MPFR_RNDN);
   if (strcmp (s, "12") || (e != 1000))
     {
@@ -1014,10 +1012,16 @@ check_large (void)
   mpfr_free_str (s);
 
   mpfr_set_nan (x);
+  mpfr_clear_flags ();
   s = mpfr_get_str (NULL, &e, 10, 1000, x, MPFR_RNDN);
   if (strcmp (s, "@NaN@"))
     {
-      printf ("Error for NaN\n");
+      printf ("Error for NaN (incorrect string)\n");
+      exit (1);
+    }
+  if (__gmpfr_flags != MPFR_FLAGS_NAN)
+    {
+      printf ("Error for NaN (incorrect flags)\n");
       exit (1);
     }
   mpfr_free_str (s);
@@ -1081,6 +1085,8 @@ check_special (int b, mpfr_prec_t p)
   int r;
   size_t m;
 
+  mpfr_init2 (x, p);
+
   /* check for invalid base */
   if (mpfr_get_str (s, &e, 1, 10, x, MPFR_RNDN) != NULL)
     {
@@ -1094,16 +1100,15 @@ check_special (int b, mpfr_prec_t p)
     }
 
   s2[0] = '1';
-  for (i=1; i<MAX_DIGITS+2; i++)
+  for (i = 1; i < MAX_DIGITS + 2; i++)
     s2[i] = '0';
 
-  mpfr_init2 (x, p);
   mpfr_set_ui (x, 1, MPFR_RNDN);
-  for (i=1; i<MAX_DIGITS && mpfr_mul_ui (x, x, b, MPFR_RNDN) == 0; i++)
+  for (i = 1; i < MAX_DIGITS && mpfr_mul_ui (x, x, b, MPFR_RNDN) == 0; i++)
     {
       /* x = b^i (exact) */
-      for (r = 0; r < MPFR_RND_MAX; r++)
-        for (m= (i<3)? 2 : i-1 ; (int) m <= i+1 ; m++)
+      RND_LOOP (r)
+        for (m = i < 3 ? 2 : i-1 ; (int) m <= i+1 ; m++)
           {
             mpfr_get_str (s, &e, b, m, x, (mpfr_rnd_t) r);
             /* s should be 1 followed by (m-1) zeros, and e should be i+1 */
@@ -1116,7 +1121,7 @@ check_special (int b, mpfr_prec_t p)
       if (mpfr_sub_ui (x, x, 1, MPFR_RNDN) != 0)
         break;
       /* now x = b^i-1 (exact) */
-      for (r = 0; r < MPFR_RND_MAX; r++)
+      RND_LOOP (r)
         if (i >= 2)
           {
             mpfr_get_str (s, &e, b, i, x, (mpfr_rnd_t) r);
@@ -1199,8 +1204,267 @@ check_reduced_exprange (void)
   mpfr_clear (x);
 }
 
+static void
+check_inex (void)
+{
+  mpfr_t x;
+  int i;
+
+  mpfr_init2 (x, 8);
+  mpfr_set_str (x, "0.11111111E-17", 2, MPFR_RNDN);
+  for (i = 1; i <= 25; i++)
+    {
+      char *s[2];
+      int r, inex = 0;
+
+      RND_LOOP (r)
+        {
+          mpfr_exp_t e;
+          mpfr_flags_t flags;
+
+          mpfr_clear_flags ();
+          s[r != 0] = mpfr_get_str (NULL, &e, 10, i, x, (mpfr_rnd_t) r);
+          MPFR_ASSERTN (e == -5);
+          flags = __gmpfr_flags;
+          if ((i >= 20) ^ (! mpfr_inexflag_p ()))
+            {
+              printf ("Error in check_inex on i=%d and %s\n",
+                      i, mpfr_print_rnd_mode ((mpfr_rnd_t) r));
+              printf ("Got %s\n", s[r != 0]);
+              printf ("Flags:");
+              flags_out (flags);
+              exit (1);
+            }
+          if (r != 0)
+            {
+              inex |= strcmp (s[0], s[1]) != 0;
+              mpfr_free_str (s[1]);
+            }
+        }
+
+      MPFR_ASSERTN ((i >= 20) ^ inex);
+      mpfr_free_str (s[0]);
+    }
+  mpfr_clear (x);
+}
+
+static void
+check_negative_base (void)
+{
+  mpfr_t f;
+  mpfr_exp_t e;
+  char *s, s2[16] = "7B000000000000", s3[16] = "74000000000000";
+
+  mpfr_init_set_ui (f, 123, MPFR_RNDN);
+  s = mpfr_get_str (0, &e, -16, 0, f, MPFR_RNDN);
+  MPFR_ASSERTN(strcmp (s, s2) == 0);
+  mpfr_free_str (s);
+  s = mpfr_get_str (0, &e, -17, 0, f, MPFR_RNDN);
+  MPFR_ASSERTN(strcmp (s, s3) == 0);
+  mpfr_free_str (s);
+  mpfr_clear (f);
+}
+
 #define ITER 1000
 
+static void
+coverage (void)
+{
+  mpfr_t x;
+  char s[42];
+  mpfr_exp_t e;
+  int b = 3;
+  size_t m = 40;
+
+  mpfr_init2 (x, 128);
+
+  /* exercise corner case in mpfr_get_str_aux: exact case (e < 0), where r
+     rounds to a power of 2, and f is a multiple of GMP_NUMB_BITS */
+  mpfr_set_ui_2exp (x, 1, 64, MPFR_RNDU);
+  mpfr_nextbelow (x);
+  /* x = 2^64 - 2^(-64) */
+  mpfr_get_str (s, &e, b, m, x, MPFR_RNDU);
+  /* s is the base-3 string for 6148914691236517206 (in base 10) */
+  MPFR_ASSERTN(strcmp (s, "1111222002212212010121102012021021021200") == 0);
+  MPFR_ASSERTN(e == 41);
+
+  /* exercise corner case in mpfr_get_str: input is m=0, then it is changed
+     to m=1 */
+  mpfr_set_prec (x, 1);
+  mpfr_set_ui (x, 1, MPFR_RNDN);
+  mpfr_get_str (s, &e, 2, 0, x, MPFR_RNDN);
+  MPFR_ASSERTN(strcmp (s, "1") == 0);
+  MPFR_ASSERTN(e == 1);
+  mpfr_get_str (s, &e, 2, 1, x, MPFR_RNDN);
+  MPFR_ASSERTN(strcmp (s, "1") == 0);
+  MPFR_ASSERTN(e == 1);
+
+  /* exercise corner case in mpfr_get_str: case m < g, exact <> 0,
+     nx > 2 * n, and low nx-2n limbs from xp not identically zero */
+  mpfr_set_prec (x, 129);
+  mpfr_set_str_binary (x, "0.100100111110110111100010100011011111101010110000011001000111111101100110101110110000110011E8");
+  mpfr_nextabove (x);
+  mpfr_get_str (s, &e, 10, 2, x, MPFR_RNDZ);
+  MPFR_ASSERTN(strcmp (s, "14") == 0);
+  MPFR_ASSERTN(e == 3);
+
+  mpfr_clear (x);
+}
+
+static void
+test_ndigits_aux (int b, mpfr_prec_t p, size_t expected_m)
+{
+  size_t m;
+  mpfr_exp_t old_emin, old_emax, e[] = { MPFR_EMIN_MIN, 0, MPFR_EMAX_MAX };
+  mpfr_flags_t flags;
+  int i;
+
+  old_emin = mpfr_get_emin ();
+  old_emax = mpfr_get_emax ();
+
+  i = randlimb () % (numberof (e) + 1);
+  if (i < numberof (e))
+    {
+      set_emin (e[i]);
+      set_emax (e[i]);
+    }
+
+  __gmpfr_flags = flags = randlimb () & MPFR_FLAGS_ALL;
+  m = mpfr_get_str_ndigits (b, p);
+  MPFR_ASSERTN (m == expected_m);
+  MPFR_ASSERTN (__gmpfr_flags == flags);
+
+  set_emin (old_emin);
+  set_emax (old_emax);
+}
+
+/* test of mpfr_get_str_ndigits */
+static void
+test_ndigits (void)
+{
+  mpfr_prec_t p;
+
+  /* for b=2, we have 1 + ceil((p-1)*log(2)/log(b)) = p */
+  for (p = MPFR_PREC_MIN; p <= 1024; p++)
+    test_ndigits_aux (2, p, p);
+
+  /* for b=4, we have 1 + ceil((p-1)*log(2)/log(b)) = 1 + ceil((p-1)/2)
+     = 1 + floor(p/2) */
+  for (p = MPFR_PREC_MIN; p <= 1024; p++)
+    test_ndigits_aux (4, p, 1 + (p / 2));
+
+  /* for b=8, we have 1 + ceil((p-1)*log(2)/log(b)) = 1 + ceil((p-1)/3)
+     = 1 + floor((p+1)/3) */
+  for (p = MPFR_PREC_MIN; p <= 1024; p++)
+    test_ndigits_aux (8, p, 1 + ((p + 1) / 3));
+
+  /* for b=16, we have 1 + ceil((p-1)*log(2)/log(b)) = 1 + ceil((p-1)/4)
+     = 1 + floor((p+2)/4) */
+  for (p = MPFR_PREC_MIN; p <= 1024; p++)
+    test_ndigits_aux (16, p, 1 + ((p + 2) / 4));
+
+  /* for b=32, we have 1 + ceil((p-1)*log(2)/log(b)) = 1 + ceil((p-1)/5)
+     = 1 + floor((p+3)/5) */
+  for (p = MPFR_PREC_MIN; p <= 1024; p++)
+    test_ndigits_aux (32, p, 1 + ((p + 3) / 5));
+
+  /* error < 1e-3 */
+  test_ndigits_aux (57, 35, 8);
+
+  /* error < 1e-4 */
+  test_ndigits_aux (31, 649, 133);
+
+  /* error < 1e-5 */
+  test_ndigits_aux (43, 5041, 931);
+
+  /* error < 1e-6 */
+  test_ndigits_aux (41, 17771, 3319);
+
+  /* 20th convergent of log(2)/log(3) */
+  test_ndigits_aux (3, 630138897, 397573381);
+
+#if MPFR_PREC_BITS >= 64
+  /* 21st convergent of log(2)/log(3) */
+  test_ndigits_aux (3, 9809721694, 6189245292);
+
+  /* 22nd convergent of log(2)/log(3) */
+  test_ndigits_aux (3, 10439860591, 6586818672);
+
+  /* 23rd convergent of log(2)/log(3) */
+  test_ndigits_aux (3, 103768467013, 65470613322);
+
+  /* 24th convergent of log(2)/log(3) */
+  test_ndigits_aux (3, 217976794617, 137528045314);
+
+  test_ndigits_aux (3, 1193652440098, 753110839882);
+
+  test_ndigits_aux (3, 683381996816440, 431166034846569);
+
+  test_ndigits_aux (7, 186564318007, 66455550933);
+#endif
+}
+
+/* check corner cases: radix-b number near from b^e converted to mpfr_t
+   and then back to radix b */
+static void
+check_corner (void)
+{
+  int b;             /* external radix */
+  mpfr_exp_t e;      /* external exponent */
+  mpfr_prec_t oprec; /* external precision */
+  mpfr_prec_t iprec; /* internal precision */
+#define MAX_OPREC 100
+  char *s, *t;
+  int i, ret;
+  mpfr_exp_t f;
+
+  for (b = 2; b <= 62; b++)
+    for (e = -100; e <= 100; e++)
+      for (iprec = MPFR_PREC_MIN; iprec <= 100; iprec ++)
+        {
+          mpfr_t x, y;
+          oprec = mpfr_get_str_ndigits (b, iprec);
+          s = (char *) tests_allocate (oprec + 6);
+            /* oprec characters for the significand, 1 for the '@' sign,
+               at most 4 for the exponent (-100), and 1 for the terminating
+               '\0'. */
+          t = (char *) tests_allocate (oprec + 6);
+          mpfr_init2 (x, iprec);
+          mpfr_init2 (y, iprec);
+          /* set s to 1000...000Ee */
+          s[0] = '1';
+          for (i = 1; i < oprec; i++)
+            s[i] = '0';
+          s[oprec] = '@';
+          ret = sprintf (s + oprec + 1, "%ld", (long) e);
+          MPFR_ASSERTN(ret <= 4);
+          /* sprintf prints the terminating null byte */
+          ret = mpfr_set_str (x, s, b, MPFR_RNDN);
+          MPFR_ASSERTN(ret == 0);
+          mpfr_get_str (t, &f, b, 0, x, MPFR_RNDN);
+          MPFR_ASSERTN(strlen (t) == oprec);
+          t[oprec] = '@';
+          ret = sprintf (t + oprec + 1, "%ld", (long) (f - oprec));
+          MPFR_ASSERTN(ret <= 4);
+          ret = mpfr_set_str (y, t, b, MPFR_RNDN);
+          MPFR_ASSERTN(ret == 0);
+          if (!mpfr_equal_p (x, y))
+            {
+              printf ("mpfr_set_str o mpfr_get_str <> Id for b=%d\n", b);
+              printf ("x="); mpfr_dump (x);
+              t[oprec] = '\0';
+              printf ("mpfr_get_str converted to 0.%s@%ld\n", t, (long) f);
+              printf ("mpfr_set_str converted to:\n");
+              printf ("y="); mpfr_dump (y);
+              exit (1);
+            }
+          mpfr_clear (x);
+          mpfr_clear (y);
+          tests_free (s, oprec + 6);
+          tests_free (t, oprec + 6);
+        }
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -1215,6 +1479,9 @@ main (int argc, char *argv[])
 
   tests_start_mpfr ();
 
+  check_corner ();
+  test_ndigits ();
+  coverage ();
   check_small ();
 
   check_special (2, 2);
@@ -1231,8 +1498,9 @@ main (int argc, char *argv[])
       m = 2 + (randlimb () % (MAX_DIGITS - 1));
       mpfr_urandomb (x, RANDS);
       e = (mpfr_exp_t) (randlimb () % 21) - 10;
-      mpfr_set_exp (x, (e == -10) ? mpfr_get_emin () :
-                    ((e == 10) ? mpfr_get_emax () : e));
+      if (!MPFR_IS_ZERO(x))
+        mpfr_set_exp (x, (e == -10) ? mpfr_get_emin () :
+                      ((e == 10) ? mpfr_get_emax () : e));
       b = 2 + (randlimb () % 35);
       r = RND_RAND ();
       mpfr_get_str (s, &f, b, m, x, r);
@@ -1262,6 +1530,8 @@ main (int argc, char *argv[])
 
   check_bug_base2k ();
   check_reduced_exprange ();
+  check_inex ();
+  check_negative_base ();
 
   tests_end_mpfr ();
   return 0;
