@@ -1,6 +1,6 @@
 /* Test file for mpfr_log2.
 
-Copyright 2001-2002, 2004, 2006-2022 Free Software Foundation, Inc.
+Copyright 2001-2002, 2004, 2006-2023 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -30,37 +30,40 @@ static void
 special (void)
 {
   mpfr_t x;
-  int inex;
+  int inex, r;
 
   mpfr_init (x);
 
-  mpfr_set_nan (x);
-  inex = mpfr_log2 (x, x, MPFR_RNDN);
-  MPFR_ASSERTN (mpfr_nan_p (x) && inex == 0);
+  RND_LOOP (r)
+    {
+      mpfr_set_nan (x);
+      inex = mpfr_log2 (x, x, (mpfr_rnd_t) r);
+      MPFR_ASSERTN (mpfr_nan_p (x) && inex == 0);
 
-  mpfr_set_inf (x, -1);
-  inex = mpfr_log2 (x, x, MPFR_RNDN);
-  MPFR_ASSERTN (mpfr_nan_p (x) && inex == 0);
+      mpfr_set_inf (x, -1);
+      inex = mpfr_log2 (x, x, (mpfr_rnd_t) r);
+      MPFR_ASSERTN (mpfr_nan_p (x) && inex == 0);
 
-  mpfr_set_inf (x, 1);
-  inex = mpfr_log2 (x, x, MPFR_RNDN);
-  MPFR_ASSERTN (mpfr_inf_p (x) && mpfr_sgn (x) > 0 && inex == 0);
+      mpfr_set_inf (x, 1);
+      inex = mpfr_log2 (x, x, (mpfr_rnd_t) r);
+      MPFR_ASSERTN (mpfr_inf_p (x) && mpfr_sgn (x) > 0 && inex == 0);
 
-  mpfr_set_ui (x, 0, MPFR_RNDN);
-  inex = mpfr_log2 (x, x, MPFR_RNDN);
-  MPFR_ASSERTN (mpfr_inf_p (x) && mpfr_sgn (x) < 0 && inex == 0);
-  mpfr_set_ui (x, 0, MPFR_RNDN);
-  mpfr_neg (x, x, MPFR_RNDN);
-  inex = mpfr_log2 (x, x, MPFR_RNDN);
-  MPFR_ASSERTN (mpfr_inf_p (x) && mpfr_sgn (x) < 0 && inex == 0);
+      mpfr_set_ui (x, 0, MPFR_RNDN);
+      inex = mpfr_log2 (x, x, (mpfr_rnd_t) r);
+      MPFR_ASSERTN (mpfr_inf_p (x) && mpfr_sgn (x) < 0 && inex == 0);
+      mpfr_set_ui (x, 0, MPFR_RNDN);
+      mpfr_neg (x, x, MPFR_RNDN);
+      inex = mpfr_log2 (x, x, (mpfr_rnd_t) r);
+      MPFR_ASSERTN (mpfr_inf_p (x) && mpfr_sgn (x) < 0 && inex == 0);
 
-  mpfr_set_si (x, -1, MPFR_RNDN);
-  inex = mpfr_log2 (x, x, MPFR_RNDN);
-  MPFR_ASSERTN (mpfr_nan_p (x) && inex == 0);
+      mpfr_set_si (x, -1, MPFR_RNDN);
+      inex = mpfr_log2 (x, x, (mpfr_rnd_t) r);
+      MPFR_ASSERTN (mpfr_nan_p (x) && inex == 0);
 
-  mpfr_set_si (x, 1, MPFR_RNDN);
-  inex = mpfr_log2 (x, x, MPFR_RNDN);
-  MPFR_ASSERTN (mpfr_cmp_ui (x, 0) == 0 && MPFR_IS_POS(x) && inex == 0);
+      mpfr_set_si (x, 1, MPFR_RNDN);
+      inex = mpfr_log2 (x, x, (mpfr_rnd_t) r);
+      MPFR_ASSERTN (MPFR_IS_ZERO (x) && MPFR_IS_POS (x) && inex == 0);
+    }
 
   mpfr_clear (x);
 }
@@ -75,6 +78,8 @@ main (int argc, char *argv[])
   test_generic (MPFR_PREC_MIN, 100, 30);
 
   data_check ("data/log2", mpfr_log2, "mpfr_log2");
+  bad_cases (mpfr_log2, mpfr_exp2, "mpfr_log2",
+             256, -30, 30, 4, 128, 800, 50);
 
   tests_end_mpfr ();
   return 0;
