@@ -1,6 +1,6 @@
 /* Test file for mpfr_version.
 
-Copyright 2004-2022 Free Software Foundation, Inc.
+Copyright 2004-2023 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -387,14 +387,6 @@ main (void)
 #endif
           );
 
-  (puts) ("[tversion] _mulx_u64 = "
-#if defined(HAVE_MULX_U64)
-          "yes"
-#else
-          "no"
-#endif
-          );
-
   if (strcmp (mpfr_buildopt_tune_case (), MPFR_TUNE_CASE) != 0)
     {
       printf ("ERROR! mpfr_buildopt_tune_case() and MPFR_TUNE_CASE"
@@ -478,7 +470,8 @@ main (void)
   e = mpfr_get_emin_min ();
   if (e != MPFR_EMIN_MIN)
     {
-      printf ("ERROR! mpfr_get_emin_min != MPFR_EMIN_MIN (%ld vs %ld)\n",
+      printf ("ERROR! mpfr_get_emin_min != MPFR_EMIN_MIN (%"
+              MPFR_EXP_FSPEC "d vs %" MPFR_EXP_FSPEC "d)\n",
               (mpfr_eexp_t) e, (mpfr_eexp_t) MPFR_EMIN_MIN);
       err = 1;
     }
@@ -486,7 +479,8 @@ main (void)
   e = mpfr_get_emax_max ();
   if (e != MPFR_EMAX_MAX)
     {
-      printf ("ERROR! mpfr_get_emax_max != MPFR_EMAX_MAX (%ld vs %ld)\n",
+      printf ("ERROR! mpfr_get_emax_max != MPFR_EMAX_MAX (%"
+              MPFR_EXP_FSPEC "d vs %" MPFR_EXP_FSPEC "d)\n",
               (mpfr_eexp_t) e, (mpfr_eexp_t) MPFR_EMAX_MAX);
       err = 1;
     }
